unit BlobEffect;

interface

Uses Globals,SysUtils,dglOpenGL;

Const xRows =32;
      yRows =16;
      numVertices = 2*yrows*xrows+xrows+2;   //842;
      numTriangles = 2*xrows+yrows*xrows*4;  //1680;
      PI2  = 6.28318530717958647692528;  // 2*PI

type TTriangle = record
       V : Array[0..2] of Integer;     // Vertices
       T : Array[0..2] of TTexCoord;   // Texture coordinates
     end;
     TBlob = Record
       vertex : Array[0..numVertices-1] of TVertex;
       Triangle : Array[0..numTriangles-1] of TTriangle;
     end;
     TText = record
      strText : string;
      TextColorRGB : GLfloat;
      StartTextTime , EndTextTime : integer;
      TransX, TransY,TransZ : GLfloat;
     end;

  procedure CreateSphere(var obj : TBlob);
  procedure RenderBlob(const obj : TBlob);
  procedure addFadeText(istrText : string; StartTextTime, EndTextTime : integer;TransX, TransY,TransZ : GLfloat);
  procedure drawBlob;
  procedure initBlob;

implementation

var Blob : TBlob;
    sphere : TBlob;
    LU : Array[0..numVertices-1] of TVertex;
    arrFadeText : array of TText;
{------------------------------------------------------------------}
{  Function to create a sphere                                     }
{------------------------------------------------------------------}
procedure CreateSphere(var obj : TBlob);
const R = 10;
      yRing =yRows*4 + 4;
      uTex = 3.0 / (xRows);
      vTex = 3.0 / (3+2*yRows);
var X, Y, I, J : Integer;
    yLevel, Radius : glFloat;
    offset : Integer;
begin
  // Top center
  obj.vertex[0].x := 0;
  obj.vertex[0].z := 0;
  obj.vertex[0].y := -r;
  offset := 1;

  // Top half and center
  for Y :=0 to yRows do
  begin
    yLevel := -r*cos(PI2*(y+1) / yring);
    radius := r*sin(PI2*(y+1) / yring);

    for X :=0 to xRows-1 do
    begin
      obj.vertex[offset].x := radius * sin(PI2*x / xRows);
      obj.vertex[offset].z := radius * cos(PI2*x / xRows);
      obj.vertex[offset].y := yLevel;
      Inc(offset);
    end;
  end;

  // Bottom half
  for Y :=0 to yRows-1 do
  begin
    yLevel := r*sin(PI2*(y+1) / yring);
    radius := r*cos(PI2*(y+1) / yring);

    for X :=0 to xRows-1 do
    begin
      obj.vertex[offset].x := radius * sin(PI2*x / xRows);
      obj.vertex[offset].z := radius * cos(PI2*x / xRows);
      obj.vertex[offset].y := ylevel;
      Inc(offset);
    end;
  end;

  // Bottom center
  obj.vertex[offset].x := 0;
  obj.vertex[offset].z := 0;
  obj.vertex[offset].y := r;

  with Obj do
  begin
    for i :=0 to xRows-1 do
    begin
      Triangle[i].v[0] := 0;
      Triangle[i].v[1] := (i+1) MOD xRows +1;
      Triangle[i].v[2] := (i+1);

      Triangle[i].T[0].u := 0.5;
      Triangle[i].T[0].v := 0.0;
      Triangle[i].T[1].u := (i+1)*uTex;
      Triangle[i].T[1].v := vTex;
      Triangle[i].T[2].u := i*uTex;
      Triangle[i].T[2].v := vTex;
    end;

    for J :=0 to yRows*2 -1 do
    begin
      for I :=0 to xRows-1 do
      begin
        offset := xRows+(i+j*xRows)*2;
        Triangle[offset].v[0] := j*xRows+1 + i;
        Triangle[offset].v[1] := j*xRows+1 + (i+1) MOD xRows;
        Triangle[offset].v[2] := j*xRows+1 + i +xRows;

        Triangle[offset].T[0].u := i*uTex;
        Triangle[offset].T[0].v := (1+j) * vTex;
        Triangle[offset].T[1].u := (1+i) * uTex;
        Triangle[offset].T[1].v := (1+j) * vTex;
        Triangle[offset].T[2].u := i     * uTex;
        Triangle[offset].T[2].v := (2+j) * vTex;

        offset := xRows+(i+j*xRows)*2 + 1;
        Triangle[offset].v[0] := j*xRows+1+ (i+1) MOD xRows;
        Triangle[offset].v[1] := j*xRows+1+ (i+1) MOD xRows + xRows;
        Triangle[offset].v[2] := j*xRows+1+   i+xRows;

        Triangle[offset].T[0].u := (i+1) * uTex;
        Triangle[offset].T[0].v := (1+j) * vTex;
        Triangle[offset].T[1].u := (i+1) * uTex;
        Triangle[offset].T[1].v := (2+j) * vTex;
        Triangle[offset].T[2].u := i     * uTex;
        Triangle[offset].T[2].v := (2+j) * vTex;
      end;

      for i := 0 to xRows-1 do
      begin
        offset := xRows+xRows*2*yRows*2 + i;
        Triangle[offset].v[0] := 2*yRows*xRows+1+i;
        Triangle[offset].v[1] := 2*yRows*xRows+1+ (i+1) MOD xRows;
        Triangle[offset].v[2] := 2*yRows*xRows+1+xRows;

        Triangle[offset].T[0].u := i           * uTex;
        Triangle[offset].T[0].v := (yRows*2+1) * vTex;
        Triangle[offset].T[1].u := (i+1)       * uTex;
        Triangle[offset].T[1].v := (yRows*2+1) * vTex;
        Triangle[offset].T[2].u := 0.5;
        Triangle[offset].T[2].v := 1.0;
      end;
    end;
  end;

end;


{------------------------------------------------------------------}
{  Function to draw the blob (warped sphere)                       }
{------------------------------------------------------------------}
procedure RenderBlob(const obj : TBlob);
var I : Integer;
begin
  glBegin (GL_TRIANGLES);
    begin
      for I :=0 to numTriangles-1 do
      begin
        with obj.Triangle[i] do
        begin

          glTexCoord2f(T[0].u, T[0].v);
          glVertex3f(obj.vertex[v[0]].x, obj.vertex[v[0]].y, obj.vertex[v[0]].z);

          glTexCoord2f (T[1].u, T[1].v);
          glVertex3f (obj.vertex[v[1]].x, obj.vertex[v[1]].y, obj.vertex[v[1]].z);

          glTexCoord2f (T[2].u, T[2].v);
          glVertex3f (obj.vertex[v[2]].x, obj.vertex[v[2]].y, obj.vertex[v[2]].z);
        end;
      end;
    end;
  glEnd();

  if ElapsedTime > DEMO_END then
    Inc(Stage);
end;

{------------------------------------------------------------------}
{  Draw an amount of polygons equal to the length of strText with chars}
{------------------------------------------------------------------}
procedure glImgWrite(strText : string);
var I, intAsciiCode : integer;
    imgcharWidth,imgcharPosX : GLfloat;
begin

  imgcharWidth := 1.0/66;
  strText := UpperCase(strText);

  for I := 1 to length(strText) do
  begin
  //66 chars
    if ord(strText[I]) > 31 then
    begin
      intAsciiCode := ord(strText[I]) - 32;
      imgcharPosX := length(strText)/2*0.08-length(strText)*0.08 + (i-1) * 0.08;
      glBegin(GL_QUADS);

        glTexCoord2f(imgcharWidth*intAsciiCode, 0.0);
        glVertex3f(-0.04+imgcharPosX, -0.04,  0.0);

        glTexCoord2f(imgcharWidth*intAsciiCode+imgcharWidth, 0.0);
        glVertex3f( 0.04+imgcharPosX, -0.04,  0.0);

        glTexCoord2f(imgcharWidth*intAsciiCode+imgcharWidth, 1.0);
        glVertex3f( 0.04+imgcharPosX,  0.04,  0.0);

        glTexCoord2f(imgcharWidth*intAsciiCode, 1.0);
        glVertex3f(-0.04+imgcharPosX,  0.04,  0.0);

      glEnd;
    end;
  end;
end;

{------------------------------------------------------------------}
{  Function to draw the actual scene                               }
{------------------------------------------------------------------}
procedure drawBlob;
var I : Integer;
    C : glFloat;
    DemoTime : Integer;
begin
  DemoTime :=ElapsedTime - BLOB_START;

  // --- Draw Background --- //
  glPushMatrix;
    C :=DemoTime/1000;
    glTranslatef(0.0,0.0,-10);
    glColor3f(1.0, 1.0, 1.0);

    glRotatef(cos(C)*30, 1, 0, 0);
    glRotatef(cos(C)*10, 0, 1, 0);
    glRotatef(-30*C,     0, 0, 1);

    // Draw Background
    glPushMatrix();
      glDisable(GL_DEPTH_TEST);
      glTranslatef(0, 0, -35);
      glScalef(20, 20, 20);

      // Water background
      glActiveTextureARB(GL_TEXTURE1_ARB);
      glDisable( GL_TEXTURE_2D );
      glDisable(GL_TEXTURE_GEN_S);                                   // Enable S texture coordinate generation
      glDisable(GL_TEXTURE_GEN_T);                                   // Enable T texture coordinate generation
      glActiveTextureARB(GL_TEXTURE0_ARB);
      glBindTexture(GL_TEXTURE_2D, WaterTex);
      RenderBlob(sphere);

      glEnable(GL_BLEND);
      glRotatef(-15 * C ,0.0,0.0,1.0);
      RenderBlob(sphere);
      glDisable(GL_BLEND);

      glEnable(GL_DEPTH_TEST);
    glPopMatrix;
  glPopMatrix;
  // Fade the sequence in
  if DemoTime < 1000 then
  begin
    glEnable(GL_BLEND);
    glPushMatrix;

      C :=1-DemoTime/1000;
      glColor3f(C, C, C);
      glBindTexture(GL_TEXTURE_2D, WhiteTex);
      glBegin(GL_QUADS);
        glTexCoord2f(0, 0);  glVertex3f(-100, -100, -1.8);
        glTexCoord2f(1, 0);  glVertex3f( 100, -100, -1.8);
        glTexCoord2f(1, 1);  glVertex3f( 100,  100, -1.8);
        glTexCoord2f(0, 1);  glVertex3f(-100,  100, -1.8);
      glEnd;

    glPopMatrix;

    glDisable(GL_BLEND);
  end;

  // Fade the sequence out
  if ElapsedTime > FIREWATER_START - 800 then
  begin
    C := (FIREWATER_START - ElapsedTime) / 800;
    glTranslatef(0.0,0.0,-2);
    glDisable(GL_TEXTURE_2D);
    glDisable(GL_BLEND);
    glEnable(GL_DEPTH_TEST);
    glColor3f(0, 0, 0);
    glBegin(GL_QUADS);
      glVertex3f(-1.2,  C , 0);
      glVertex3f( 1.2,  C , 0);
      glVertex3f( 1.2, C+1, 0);
      glVertex3f(-1.2, C+1, 0);

      glVertex3f(-1.2, -1-C, 0);
      glVertex3f( 1.2, -1-C, 0);
      glVertex3f( 1.2,  -C , 0);
      glVertex3f(-1.2,  -C , 0);
    glEnd();
    glColor3f(1, 1, 1);
    glEnable(GL_TEXTURE_2D);
  end;



  glPushMatrix;
    glColor3f(1, 1, 1);
    glEnable(GL_ALPHA_TEST);
    glAlphaFunc(GL_GREATER, 0.4);
    glEnable(GL_BLEND);
    glTranslatef(0.0,0.0,-3);

    glBindTexture(GL_TEXTURE_2D, FontTex);  // Bind the Texture to the object
    glActiveTextureARB(GL_TEXTURE1_ARB);
    glBindTexture(GL_TEXTURE_2D, MultiTexture[g_VolitionLogo.CurrentTextureIndex]);
    glEnable( GL_TEXTURE_2D );
    glTexGenfv(GL_S, GL_SPHERE_MAP, 0);                           // All S texture coordinates will be generated for a sphere
    glTexGenfv(GL_T, GL_SPHERE_MAP, 0);                           // All T texture coordinates will be generated for a sphere
    glEnable(GL_TEXTURE_GEN_S);                                   // Enable S texture coordinate generation
    glEnable(GL_TEXTURE_GEN_T);                                   // Enable T texture coordinate generation
    //Display text
    for I := 0 to length(arrFadeText)-1 do
    begin
      with arrFadeText[I] do
      begin
        if (DemoTime > StartTextTime) and (DemoTime < EndTextTime) then
        begin
          if DemoTime > EndTextTime-800 then
          begin
          // Fade to black
            TextColorRGB := TextColorRGB - 0.02;
          end
          else
          begin
            // Fade to white
            if TextColorRGB < 1.0 then
               TextColorRGB := TextColorRGB + 0.01;
          end;
          //Draw Text line
          glPushMatrix;
            glColor3f(TextColorRGB,TextColorRGB,TextColorRGB);
            glTranslatef(TransX,TransY,TransZ);
            glImgWrite(strText);
          glPopMatrix;

        end;
      end;
    end;

    glDisable(GL_BLEND);
    glDisable(GL_ALPHA_TEST);

  glPopMatrix;



  if ElapsedTime > FIREWATER_START then
  begin
    glActiveTextureARB(GL_TEXTURE1_ARB);
    glDisable( GL_TEXTURE_2D );
    glDisable(GL_TEXTURE_GEN_S);                                   // Enable S texture coordinate generation
    glDisable(GL_TEXTURE_GEN_T);                                   // Enable T texture coordinate generation
    glActiveTextureARB(GL_TEXTURE0_ARB);
    Inc(Stage);
  end;
end;

{------------------------------------------------------------------}
{  Add Text to a fadeText array                                    }
{------------------------------------------------------------------}
procedure addFadeText(istrText : string; StartTextTime, EndTextTime : integer;TransX, TransY,TransZ : GLfloat);
begin
   setLength(arrFadeText,length(arrFadeText)+1);
   arrFadeText[length(arrFadeText)-1].strText  := istrText;
   arrFadeText[length(arrFadeText)-1].TextColorRGB := 0.0;
   arrFadeText[length(arrFadeText)-1].StartTextTime := StartTextTime;
   arrFadeText[length(arrFadeText)-1].EndTextTime := EndTextTime;
   arrFadeText[length(arrFadeText)-1].TransX := TransX;
   arrFadeText[length(arrFadeText)-1].TransY := TransY;
   arrFadeText[length(arrFadeText)-1].TransZ := TransZ;
end;

{------------------------------------------------------------------}
{  Init                                                            }
{------------------------------------------------------------------}
procedure initBlob;
var I : Integer;
begin

  setLength(arrFadeText,length(arrFadeText));
  addFadeText('Lift Me Up Into The Arctic Sky',2000,5000,0.0,0.5,-0.1);
  addFadeText('Over The Ocean A Thousand Miles Wide.',5001,8000,0.0,0.3,-0.1);
  addFadeText('Drop Me Down Into The Frozen Sea',8001,11000,0.0,0.2,-0.1);
  addFadeText('The Sorrowful Angel Will Follow Me.',11000,14000,0.2,0.0,-0.3);
  addFadeText('Under The Ice The Colors Bleed To One',14000,17000,0.0,-0.1,-0.3);
  addFadeText('An Amalgamation Of',17001,27000,0.0,0.0,-0.1);
  addFadeText('The Moon',18001,21000,0.0,-0.2,-0.1);
  addFadeText('The Stars',21001,24000,0.0,-0.2,-0.1);
  addFadeText('And The Sun.',24001,27000,0.0,-0.2,-0.1);
  addFadeText('Author Unknown',27001,30000,0.0,-0.2,-0.1);

  CreateSphere(blob);
  Sphere :=Blob;
  for I :=0 to numVertices-1 do
  begin
    LU[i].X := ArcTan(sphere.vertex[i].x, sphere.vertex[i].y) * 5;
    LU[i].Y := ArcTan(sphere.vertex[i].x, sphere.vertex[i].z) * 6;
    LU[i].Z := ArcTan(sphere.vertex[i].y, sphere.vertex[i].z) * 8;
  end;
end;

end.
