unit BlobTunnel;

interface

  procedure initTunnel;
  procedure drawTunnel;
  procedure drawBanner;

implementation

Uses dglOpenGL, BlobEffect, Globals;

const TUNNEL_SPEED = 1/60;
      PI_8 = 0.3926990816987;    // PI/8

var Blob    : TBlob;
    Sphere  : TBlob;
    Tunnels : Array[0..64, 0..32] of TVertex;
    LU : Array[0..numVertices-1] of TVertex;
    RadialCount : integer = 0;
    FadeColor : GLfloat = 0.0;
    RotateAngle : GLfloat = 0.0;
    CurrentPosFire : integer = 1;
    CurrentPosWater : integer = 10;
    CurrentPosWind : integer = 20;
    CurrentPosEarth : integer = 30;

{------------------------------------------------------------------}
{  Draw Elements banner                                            }
{------------------------------------------------------------------}
procedure drawBanner();
begin
  glTranslatef(-0.1,0.1,0.1);
  glBindTexture(GL_TEXTURE_2D, ElementWeb);
  glBegin(GL_QUADS);
    glTexCoord2f(0, 0); glVertex3f(-0.5,-0.5, 0);
    glTexCoord2f(1, 0); glVertex3f( 0.5,-0.5, 0);
    glTexCoord2f(1, 1); glVertex3f( 0.5, 0.5, 0);
    glTexCoord2f(0, 1); glVertex3f(-0.5, 0.5, 0);
  glEnd();
end;

{------------------------------------------------------------------}
{  Draw Element moveing on array vCurve                            }
{------------------------------------------------------------------}
procedure DrawElement(elementPos : integer;CRed,CBlue,CGreen : GLfloat; boundtexture : GLuint);
begin
  glPushMatrix;

    glTranslatef(vCurves[elementPos].X-2.0,vCurves[elementPos].Y-2.0,vCurves[elementPos].Z-7.0);
    glColor3f(CRed,CBlue,CGreen);
    glBindTexture(GL_TEXTURE_2D, boundtexture);
    glRotatef(RotateAngle,1.0,0.0,0.0);
    glBegin(GL_QUADS);
      glTexCoord2f(0, 0); glVertex3f(-0.15,-0.15, 0);
      glTexCoord2f(1, 0); glVertex3f( 0.15,-0.15, 0);
      glTexCoord2f(1, 1); glVertex3f( 0.15, 0.15, 0);
      glTexCoord2f(0, 1); glVertex3f(-0.15, 0.15, 0);
    glEnd();

  glPopMatrix;
end;


{------------------------------------------------------------------}
{  Draw element at current position on vCurves Array               }
{------------------------------------------------------------------}
procedure DrawElementAtPos(CurrentPos : integer; ElementTexture : glUint);
begin
    DrawElement(CurrentPos,1.0,1.0,1.0,ElementTexture);

    if CurrentPos = 1 then
       DrawElement(High(vCurves),0.8,0.8,0.8,ElementTexture)
    else
       DrawElement(CurrentPos-1,0.8,0.8,0.8,ElementTexture);

    if CurrentPos = 2 then
       DrawElement(High(vCurves),0.6,0.6,0.6,ElementTexture)
    else
       DrawElement(CurrentPos-2,0.6,0.6,0.6,ElementTexture);

    if CurrentPos = 3 then
       DrawElement(High(vCurves),0.4,0.4,0.4,ElementTexture)
    else
       DrawElement(CurrentPos-3,0.4,0.4,0.4,ElementTexture);
end;

{------------------------------------------------------------------}
{  Draws the tunnel with the blob appearing in a window            }
{------------------------------------------------------------------}
procedure drawTunnel();
var I, J : Integer;
    C,J1, J2 : glFloat;
    X, Y : glFloat;
    Angle : glFloat;
    A1, A2, A3, A4 : glFloat;
    DemoTime : Integer;

begin
  DemoTime :=ElapsedTime - TUNNEL_START;

  // --- Drawing the tunnel --- //
  glDisable(GL_BLEND);
  glPushMatrix();
    glTranslatef(0.0,0.0,-2);

    Angle := DemoTime/14;

    //--- Outside tunnel ---//
    // setup tunnel coordinates
    A1 := sin(Angle/27);
    A2 := cos(Angle/33);
    A3 := cos(Angle/13);
    A4 := sin(Angle/17);
    for I :=0 to 16 do
    begin
      X :=cos(PI_8*I);
      Y :=sin(PI_8*I);
      for J :=0 to 32 do
      begin
        Tunnels[I, J].X :=(3 - J/12)*X + 2.0*sin((Angle+2*j)/27) + cos((Angle+2*j)/13) - 2*A1 - A3;
        Tunnels[I, J].Y :=(3 - J/12)*Y + 2.5*cos((Angle+2*j)/33) + sin((Angle+2*j)/17) - 2*A2 - A4;
        Tunnels[I, J].Z :=-J;
      end;
    end;

    // draw tunnel and fade out last few
    glBindTexture(GL_TEXTURE_2D, WaterTex);
    For J :=0 to 31 do
    begin
      J1 :=J/32 + Angle*TUNNEL_SPEED;        // precalculate texture v coords for speed
      J2 :=(J+1)/32 + Angle*TUNNEL_SPEED;

      // near the end of the tunnel, fade the effect away
      if J > 24 then
        C :=1.0-(J-24)/10
      else
        C :=1.0;

      // fade in tunnel
      if DemoTime < 500 then
        C :=C*DemoTime/500;

      glColor3f(C, C, C);

      glBegin(GL_QUADS);
        For I :=0 to 15 do
        begin
          glTexCoord2f((I-1)/16, J1); glVertex3f(Tunnels[ I,   J ].X, Tunnels[ I,   J ].Y, Tunnels[ I,   J ].Z);
          glTexCoord2f(( I )/16, J1); glVertex3f(Tunnels[I+1,  J ].X, Tunnels[I+1,  J ].Y, Tunnels[I+1,  J ].Z);
          glTexCoord2f(( I )/16, J2); glVertex3f(Tunnels[I+1, J+1].X, Tunnels[I+1, J+1].Y, Tunnels[I+1, J+1].Z);
          glTexCoord2f((I-1)/16, J2); glVertex3f(Tunnels[ I,  J+1].X, Tunnels[ I,  J+1].Y, Tunnels[ I,  J+1].Z);
        end;
      glEnd();

    end;

    //--- Inner tunnel ---//
    // setup tunnel coordinates

    for I :=0 to 16 do
    begin
      X :=cos(PI_8*I);
      Y :=sin(PI_8*I);
      for J :=0 to 28 do
      begin
        Tunnels[I, J].X :=(2.4 - J/12)*X + 2.0*sin((Angle+2*j)/27) + cos((Angle+2*j)/13) - 2*A1 - A3;
        Tunnels[I, J].Y :=(2.4 - J/12)*Y + 2.5*cos((Angle+2*j)/33) + sin((Angle+2*j)/17) - 2*A2 - A4;
        Tunnels[I, J].Z :=-J;
      end;
    end;

    X :=DemoTime/120;
    glEnable(GL_DEPTH_TEST);
    glBindTexture(GL_TEXTURE_2D, MovingFire[g_CurrentFire]);
    glEnable(GL_BLEND);
    For J :=0 to 27 do
    begin
      J1 :=J/32 + Angle*TUNNEL_SPEED;        // precalculate texture v coords for speed
      J2 :=(J+1)/32 + Angle*TUNNEL_SPEED;

      // near the end of the tunnel, fade the effect away
      if J > 24 then
        C :=0.5-(J-24)/10
      else
        C :=0.5;

      // fade in tunnel
      if DemoTime < 500 then
        C :=C*DemoTime/500;

      glColor3f(C, C, C);

      glBegin(GL_QUADS);
        For I :=0 to 15 do
        begin
          glTexCoord2f((I-1 + X)/16, J1); glVertex3f(Tunnels[ I,   J ].X, Tunnels[ I,   J ].Y, Tunnels[ I,   J ].Z);
          glTexCoord2f(( I  + X)/16, J1); glVertex3f(Tunnels[I+1,  J ].X, Tunnels[I+1,  J ].Y, Tunnels[I+1,  J ].Z);
          glTexCoord2f(( I  + X)/16, J2); glVertex3f(Tunnels[I+1, J+1].X, Tunnels[I+1, J+1].Y, Tunnels[I+1, J+1].Z);
          glTexCoord2f((I-1 + X)/16, J2); glVertex3f(Tunnels[ I,  J+1].X, Tunnels[ I,  J+1].Y, Tunnels[ I,  J+1].Z);
        end;
      glEnd();
    end;
  glPopMatrix;

  glEnable(GL_BLEND);

  RotateAngle := RotateAngle + 0.5;

  CurrentPosFire := CurrentPosFire + 1;
  CurrentPosWind := CurrentPosWind + 1;
  CurrentPosWater := CurrentPosWater + 1;
  CurrentPosEarth := CurrentPosEarth + 1;
  
  if CurrentPosFire > High(vCurves) then
    CurrentPosFire := 1;

  if CurrentPosWind > High(vCurves) then
    CurrentPosWind := 1;

  if CurrentPosWater > High(vCurves) then
    CurrentPosWater := 1;
    
  if CurrentPosEarth > High(vCurves) then
    CurrentPosEarth := 1;
    
  glEnable(GL_ALPHA_TEST);
  glAlphaFunc(GL_GREATER, 0.4);
  glDisable(GL_DEPTH_TEST);

    DrawElementAtPos(CurrentPosFire,TgaFire);
    DrawElementAtPos(CurrentPosWind,TgaWind);
    DrawElementAtPos(CurrentPosWater,TgaWater);
    DrawElementAtPos(CurrentPosEarth,TgaEarth);

  glDisable(GL_ALPHA_TEST);

  // --- Draw the Elements AND Logo --- //
  if DemoTime > 9700 then
  begin

    glEnable(GL_DEPTH_TEST);
    glColor3f(1.0, 1.0, 1.0);

    X :=(DemoTime-9700)/800;
    if X > 1 then X :=1;
    glScalef(X, X, 1);
    glTranslatef(0.7,-0.52, -2.4);

    drawBanner();




    // Expanding frame
    if (DemoTime > 13500) AND (DemoTime < 15400) then
    begin
      for I := 0 to RadialCount do
      begin
        glColor3f(1.0/I,1.0/I,1.0/I);
        drawBanner();
      end;
      RadialCount := RadialCount + 1;
    end;

    // fade to white
    if ElapsedTime > BLOB_START - 4000 then
    begin
      glEnable(GL_BLEND);
      glTranslatef(-0.0, 0.0, 0.6);
      glBindTexture(GL_TEXTURE_2D, WhiteTex);
      FadeColor := FadeColor + 0.005;

      glColor3f(FadeColor, FadeColor, FadeColor);
      glBegin(GL_QUADS);
        glTexCoord2f(0, 0);  glVertex3f(-10, -10, 0);
        glTexCoord2f(1, 0);  glVertex3f( 10, -10, 0);
        glTexCoord2f(1, 1);  glVertex3f( 10,  10, 0);
        glTexCoord2f(0, 1);  glVertex3f(-10,  10, 0);
      glEnd;
    end;
  end;

  if ElapsedTime > BLOB_START then
  begin
    glLineWidth(1);
    Inc(Stage);
  end;
end;

{------------------------------------------------------------------}
{  Init the tunnel and blob                                        }
{------------------------------------------------------------------}
procedure initTunnel;
var I : Integer;
begin
  CreateSphere(blob);
  Sphere :=Blob;
  for I :=0 to numVertices-1 do
  begin
    LU[i].X := ArcTan(sphere.vertex[i].x, sphere.vertex[i].y) * 5;
    LU[i].Y := ArcTan(sphere.vertex[i].x, sphere.vertex[i].z) * 6;
    LU[i].Z := ArcTan(sphere.vertex[i].y, sphere.vertex[i].z) * 8;
  end;
end;

end.
