unit Globals;

interface

Uses Windows, dglOpenGL, Textures, SysUtils,Animate;


const INTRO_START  = 0;
      INTRO_TUNNEL = 27000;
      STATIC_START  = 27150;
      CD_COVER_START = 42000;
      TUNNEL_START = 56300;
      BLOB_START   = 87700;
      FIREWATER_START = 117700;
      METABALL_START = 133700;
      TENTACLE_START = 176000;
      DEMO_END = 252000;

      GRID_SIZE = 80; // For Water Effect

type TNormal = record
       X, Y, Z : glFloat;             // X, Y, Z coordinates
     end;
     TVertex = record
       X, Y, Z : glFloat;             // X, Y, Z coordinates
     end;
     TTexCoord = record
       U, V : glFloat;                // U and V Texture coordinates
     end;

//--- Textures ---//
var ProgressTex   : glUint;               // Loading Stage
    DemoNameTex   : glUint;               // Intro (Band Name)
    IntroBG       : glUint;               // Intro (Full Band Pic in negative)
    Logo          : glUint;               // Intro (Fuzzy Volition Logo) 

    Static        : glUint;               // Static

    Fire          : glUint;               // Elements
    Water         : glUint;               // Elements
    Earth         : glUint;               // Elements
    Wind          : glUint;               // Elements
    ReflectTex    : glUint;               // Elements
    ElementBG     : glUint;               // Elements CD Cover

    ElementWeb    :glUint;                // TunnelBanner
    TgaFire       :glUint;                // StrangeBlob
    TgaWater      :glUint;                // StrangeBlob
    TgaWind       :glUint;                // StrangeBlob
    TgaEarth      :glUint;                // StrangeBlob

    FontTex       :glUint;                // Volition Font

    MovingFire: array[1..8] of glUint;    // Moving Fire
    WaterTex      :glUint;                // Water for sea

    MultiTexture : array [0..31] of GLUint; // Causics

    TunnelTex     : glUint;               // Tunnel Textures
    FireTex       : glUint;               // Tunnel Textures

    WhiteTex      : glUint;               // fade between scenes

    EnviroTex     : glUint;               // Metaballs Enviromap
    MBallsBgTex   : Array[0..6] of glUint;// Metaballs Background

    BroughtBy     : glUint;               // Mystic Sound
    LogoSharp     : glUint;               // Volition Logo Sharp
    Design        : glUint;               // Bluespill
    Developent    : glUint;               // Sulaco
    Ocean         : glUint;               // Freddy Ocean Scene

    BmpParticle   : glUint;               // Particle


type TGLCoord = Record
       X, Y, Z : glFloat;
     end;

//--- Other Global variables ---//
var ElapsedTime : Integer;                // Elapsed time between frames
    Stage : Integer;                      // Stage the demo is at.
    FPSCount : Integer = 0;               // Counter for FPS

    g_backRed  : GLfloat = 0.0;
    g_backGreen : GLfloat = 0.0;
    g_backBlue : GLfloat = 1.0;
    g_modifier : integer = 1;

    g_CurrentFire : integer = 1;

    // 3D Logo Variables
    g_VolitionLogo : TAnimate;
    g_RotLogo : GLfloat;

    //water variables
    Viscosity : glFloat;
    Position : Array[0..GRID_SIZE, 0..GRID_SIZE] of glFloat;
    Velocity : Array[0..GRID_SIZE, 0..GRID_SIZE] of glFloat;

    Vertex : Array[0..GRID_SIZE, 0..GRID_SIZE] of TglCoord;
    Normals:array [0..GRID_SIZE, 0..GRID_SIZE] of TglCoord;

    vCurves : array of TVertex;            // Array for elements following path (Tunnel stage)
    vLazer : array of TVertex;             // Array for particles (Outtro stage)

    SphereQuadratic : PGLUquadricObj;	     // Storage For Our Quadratic Objects

  procedure ShowLoading;                   // Show Loading screen while images are being loaded and curves are initialized 
  procedure Init;                          // Procedure that load texures and objects
  function  ArcTan(X, Y : glFloat) : glFloat;
  procedure glBindTexture(target: GLenum; texture: GLuint); stdcall; external opengl32;

implementation



{------------------------------------------------------------------}
{  LOADING STAGE : Load all textures and objects.                  }
{------------------------------------------------------------------}
procedure LoadDemoTextures;
var i : integer;
begin
  LoadTexture('images/band.jpg', IntroBG, false);             // Intro
  LoadTexture('images/volitionName.jpg',  DemoNameTex, false);
  LoadTexture('images/volitionlogo.jpg',  Logo, false);

  LoadTexture('images/FireSphere.bmp',  Fire, false);             // Balls surrounding CD cover
  LoadTexture('images/WaterSphere.bmp',  Water, false);
  LoadTexture('images/EarthSphere.bmp',  Earth, false);
  LoadTexture('images/WindSphere.bmp',  Wind, false);
  LoadTexture('images/ReflectionBall.bmp',  ReflectTex, false);
  LoadTexture('images/elements_cd.jpg',  ElementBG, false);

  LoadTexture('images/Static.jpg', Static, false);                 // Static

  LoadTexture('images/elementsweb.jpg',  ElementWeb, false); // Tunnel
  LoadTexture('images/FireTriangle.tga',  TgaFire, false); // Tunnel
  LoadTexture('images/WaterDiamond.tga', TgaWater, false); // Tunnel
  LoadTexture('images/AirTiangle.tga', TgaWind, false); // Tunnel
  LoadTexture('images/EarthTransparent.tga', TgaEarth, false); // Tunnel

  LoadTexture('images/volitionfont.tga', FontTex, false); // Volition Font

  for i := 1 to 8 do
  begin
    LoadTexture('images/'+ IntToStr(i) +'.tga', MovingFire[i] , false); // Fire Water
  end;
  for i := 0 to 31 do
  begin
      LoadTexture('images/Caust' + IntToStr(i) + '.jpg',MultiTexture[i],false);
  end;

  LoadTexture('images/reflection.bmp', WaterTex, false); // Fire Water

  LoadTexture('images/white.bmp',  WhiteTex, false);

  LoadTexture('images/NatSoft.jpg', MBallsBgTex[0], false);   // Metaball Background Band
  LoadTexture('images/AdeBrush.jpg', MBallsBgTex[1], false);
  LoadTexture('images/FreddyPencil.jpg', MBallsBgTex[2], false);
  LoadTexture('images/NathanBass.jpg', MBallsBgTex[3], false);
  LoadTexture('images/Crowd.jpg', MBallsBgTex[4], false);
  LoadTexture('images/FullBandChrome.jpg', MBallsBgTex[5], false);
  LoadTexture('images/Ferdi.jpg', MBallsBgTex[6], false);

  LoadTexture('images/particle.bmp', BmpParticle, false);
  LoadTexture('images/ocean.jpg', Ocean, false);
  LoadTexture('images/chrome.jpg', EnviroTex, false);


  LoadTexture('images/BroughtByMST.jpg', BroughtBy, false);          // Outtro
  LoadTexture('images/DesignBPD.jpg', Design, false);
  LoadTexture('images/DevSulaco.jpg', Developent, false);
  LoadTexture('images/LogoSharp.jpg', LogoSharp, false);
end;

{------------------------------------------------------------------}
{  Add Vector to Lazer array                                       }
{------------------------------------------------------------------}
procedure addLazerPoint(X,Y,Z : GLfloat);
begin
  setLength(vLazer,high(vLazer)+2);
  vLazer[high(vLazer)].X := X;
  vLazer[high(vLazer)].Y := Y;
  vLazer[high(vLazer)].Z := Z;
end;

{------------------------------------------------------------------}
{  Init the Lazer curve array                                      }
{------------------------------------------------------------------}
procedure initLazerCurve();
begin
 setLength(vLazer,1);
 addLazerPoint(0,1,0);
 addLazerPoint(0.0788926929235458,1.0005704164505,0);
 addLazerPoint(0.157457366585732,1.00207030773163,0);
 addLazerPoint(0.235365927219391,1.00418245792389,0);
 addLazerPoint(0.312290400266647,1.00658977031708,0);
 addLazerPoint(0.387902736663818,1.00897550582886,0);
 addLazerPoint(0.461874902248383,1.01102197170258,0);
 addLazerPoint(0.533878803253174,1.0124124288559,0);
 addLazerPoint(0.603586494922638,1.01282978057861,0);
 addLazerPoint(0.670669972896576,1.01195693016052,0);
 addLazerPoint(0.73480099439621,1.00947654247284,0);
 addLazerPoint(0.795651733875275,1.00507187843323,0);
 addLazerPoint(0.852894008159638,0.998425424098968,0);
 addLazerPoint(0.906199753284454,0.989220261573791,0);
 addLazerPoint(0.955241203308105,0.977139532566071,0);
 addLazerPoint(0.999690175056457,0.961865901947021,0);
 addLazerPoint(1.03921842575073,0.943082094192505,0);
 addLazerPoint(1.07349812984467,0.920471251010895,0);
 addLazerPoint(1.10220146179199,0.893716335296631,0);
 addLazerPoint(1.125,0.862500190734863,0);
 addLazerPoint(1.14156579971313,0.826505422592163,0);
 addLazerPoint(1.15157091617584,0.785415291786194,0);
 addLazerPoint(1.15468728542328,0.738912463188171,0);
 addLazerPoint(1.1505868434906,0.68667995929718,0);
 addLazerPoint(1.13894152641296,0.628400683403015,0);
 addLazerPoint(1.11942338943481,0.563757479190826,0);
 addLazerPoint(1.09170436859131,0.492433249950409,0);
 addLazerPoint(1.05545628070831,0.414110898971558,0);
 addLazerPoint(1.01035130023956,0.32847335934639,0);
 addLazerPoint(0.956061244010925,0.235203474760056,0);
 addLazerPoint(0.892258107662201,0.133984178304672,0);
 addLazerPoint(0.818613886833191,0.0244983676820993,0);
 addLazerPoint(0.734800577163696,-0.0935710668563843,0);
 addLazerPoint(0.640490055084228,-0.220541268587112,0);
 addLazerPoint(0.535354316234589,-0.356729298830032,0);
 addLazerPoint(0.419065356254578,-0.502452254295349,0);
 addLazerPoint(0.291295081377029,-0.658027172088623,0);
 addLazerPoint(0.151715502142906,-0.823771297931671,0);
 addLazerPoint(-1.43051204304356E-6,-1.00000166893005,0);
 addLazerPoint(0,-1,0);
 addLazerPoint(-0.116316147148609,-0.838739216327667,0);
 addLazerPoint(-0.22831317782402,-0.686368346214294,0);
 addLazerPoint(-0.335826992988586,-0.54263561964035,0);
 addLazerPoint(-0.438693642616272,-0.407289654016495,0);
 addLazerPoint(-0.536749124526978,-0.280079185962677,0);
 addLazerPoint(-0.629829406738281,-0.160752311348915,0);
 addLazerPoint(-0.717770397663116,-0.0490577816963196,0);
 addLazerPoint(-0.800408184528351,0.0552559159696102,0);
 addLazerPoint(-0.877578794956207,0.15244023501873,0);
 addLazerPoint(-0.949117958545685,0.24274680018425,0);
 addLazerPoint(-1.01486194133759,0.326426982879639,0);
 addLazerPoint(-1.07464647293091,0.403732299804687,0);
 addLazerPoint(-1.12830758094788,0.474914282560348,0);
 addLazerPoint(-1.17568147182465,0.540224432945251,0);
 addLazerPoint(-1.2166041135788,0.599914312362671,0);
 addLazerPoint(-1.25091111660004,0.654235184192657,0);
 addLazerPoint(-1.27843880653381,0.703438758850098,0);
 addLazerPoint(-1.29902315139771,0.747776508331299,0);
 addLazerPoint(-1.31250011920929,0.787499904632568,0);
 addLazerPoint(-1.31870532035828,0.822860360145569,0);
 addLazerPoint(-1.3174751996994,0.854109466075897,0);
 addLazerPoint(-1.30864560604095,0.881498694419861,0);
 addLazerPoint(-1.29205238819122,0.905279517173767,0);
 addLazerPoint(-1.26753175258636,0.925703406333923,0);
 addLazerPoint(-1.23491942882538,0.943021953105927,0);
 addLazerPoint(-1.19405162334442,0.957486510276794,0);
 addLazerPoint(-1.14476406574249,0.969348669052124,0);
 addLazerPoint(-1.08689296245575,0.978859901428223,0);
 addLazerPoint(-1.02027428150177,0.986271679401398,0);
 addLazerPoint(-0.944743871688843,0.991835594177246,0);
 addLazerPoint(-0.860137760639191,0.995802938938141,0);
 addLazerPoint(-0.766291975975037,0.998425424098968,0);
 addLazerPoint(-0.663042485713959,0.999954462051392,0);
 addLazerPoint(-0.550225257873535,1.00064146518707,0);
 addLazerPoint(-0.427676260471344,1.0007381439209,0);
 addLazerPoint(-0.295231491327286,1.00049567222595,0);
 addLazerPoint(-0.1527269333601,1.00016582012177,0);
 addLazerPoint(1.43051192935673E-6,1,0);
end;

{------------------------------------------------------------------}
{  Add Vector to Element Curve array                               }
{------------------------------------------------------------------}
procedure addPoint(X,Y,Z : GLfloat);
begin
  setLength(vCurves,high(vCurves)+2);
  vCurves[high(vCurves)].X := X;
  vCurves[high(vCurves)].Y := Y;
  vCurves[high(vCurves)].Z := Z;
end;


{------------------------------------------------------------------}
{  Init Element Curve Array                                        }
{------------------------------------------------------------------}
procedure initCurve();
begin
 addPoint(-1,1,0);
 addPoint(-0.959505677223206,1.22336745262146,-0.0231843199580908);
 addPoint(-0.917043387889862,1.4205687046051,-0.0437848120927811);
 addPoint(-0.872721970081329,1.59277772903442,-0.0617938078939915);
 addPoint(-0.826651036739349,1.74116909503937,-0.077203668653965);
 addPoint(-0.778940141201019,1.86691761016846,-0.0900067389011383);
 addPoint(-0.729698181152344,1.97119688987732,-0.100195363163948);
 addPoint(-0.679034829139709,2.05518174171448,-0.10776187479496);
 addPoint(-0.627059280872345,2.12004637718201,-0.112698636949062);
 addPoint(-0.573881089687347,2.16696572303772,-0.114997997879982);
 addPoint(-0.519609272480011,2.19711327552795,-0.114652283489704);
 addPoint(-0.4643534719944,2.21166396141052,-0.111653864383698);
 addPoint(-0.408222794532776,2.21179175376892,-0.105995051562786);
 addPoint(-0.351326704025269,2.19867134094238,-0.0976682081818581);
 addPoint(-0.29377469420433,2.17347717285156,-0.0866657122969627);
 addPoint(-0.235675901174545,2.13738369941711,-0.0729798823595047);
 addPoint(-0.177139654755592,2.09156441688538,-0.0566030405461788);
 addPoint(-0.118275366723537,2.03719449043274,-0.0375275537371635);
 addPoint(-0.0591924786567688,1.97544848918915,-0.0157458018511534);
 addPoint(-2.34693288803101E-7,1.9075003862381,0.00874989759176969);
 addPoint(0.0591920763254166,1.83452427387238,0.0359672345221043);
 addPoint(0.118275076150894,1.75769484043121,0.0659138485789299);
 addPoint(0.177139386534691,1.67818653583527,0.0985973849892616);
 addPoint(0.235675647854805,1.59717357158661,0.134025484323502);
 addPoint(0.293774545192719,1.5158303976059,0.172205805778503);
 addPoint(0.351326704025269,1.43533134460449,0.213146030902863);
 addPoint(0.408222794532776,1.35685086250305,0.25685378909111);
 addPoint(0.4643534719944,1.28156316280365,0.303336709737778);
 addPoint(0.519609391689301,1.21064281463623,0.352602511644363);
 addPoint(0.573881149291992,1.14526402950287,0.404658794403076);
 addPoint(0.627059519290924,1.08660137653351,0.459513247013092);
 addPoint(0.679035067558289,1.03582894802094,0.51717346906662);
 addPoint(0.729698419570923,0.994121372699738,0.577647149562836);
 addPoint(0.778940320014954,0.962652921676636,0.640941977500916);
 addPoint(0.826651453971863,0.942597985267639,0.707065582275391);
 addPoint(0.872722327709198,0.935130834579468,0.776025593280792);
 addPoint(0.917043626308441,0.941425979137421,0.847829699516296);
 addPoint(0.959506094455719,0.962657690048218,0.92248547077179);
 addPoint(1.00000035762787,1.00000035762787,1.00000071525574);
 addPoint(1,1,1);
 addPoint(1.07891082763672,1.03745985031128,1.29736471176147);
 addPoint(1.15760326385498,1.07114744186401,1.5589736700058);
 addPoint(1.23585796356201,1.10144519805908,1.78646659851074);
 addPoint(1.31345665454864,1.12873589992523,1.98148393630981);
 addPoint(1.39018082618713,1.15340256690979,2.1456663608551);
 addPoint(1.46581137180328,1.17582738399506,2.28065323829651);
 addPoint(1.54012966156006,1.19639337062836,2.38808488845825);
 addPoint(1.61291718482971,1.21548318862915,2.46960186958313);
 addPoint(1.6839554309845,1.23347985744476,2.52684450149536);
 addPoint(1.75302529335022,1.25076544284821,2.56145215034485);
 addPoint(1.81990826129913,1.26772320270538,2.57506585121155);
 addPoint(1.88438546657562,1.2847353219986,2.56932497024536);
 addPoint(1.94623827934265,1.30218493938446,2.54587006568909);
 addPoint(2.00524854660034,1.32045483589172,2.50634217262268);
 addPoint(2.06119704246521,1.33992755413055,2.45238041877747);
 addPoint(2.11386489868164,1.36098551750183,2.38562488555908);
 addPoint(2.16303372383118,1.38401174545288,2.30771613121033);
 addPoint(2.20848512649536,1.40938901901245,2.220294713974);
 addPoint(2.25,1.4375,2.12500047683716);
 addPoint(2.28735947608948,1.46872711181641,2.02347326278687);
 addPoint(2.32034540176392,1.503453373909,1.91735351085663);
 addPoint(2.3487389087677,1.54206144809723,1.80828130245209);
 addPoint(2.37232089042664,1.58493399620056,1.69789707660675);
 addPoint(2.39087319374084,1.63245368003845,1.58784091472626);
 addPoint(2.40417695045471,1.68500328063965,1.47975289821625);
 addPoint(2.4120135307312,1.74296545982361,1.37527334690094);
 addPoint(2.41416382789612,1.80672299861908,1.27604234218597);
 addPoint(2.41040968894958,1.87665855884552,1.18370008468628);
 addPoint(2.40053200721741,1.95315480232239,1.09988677501678);
 addPoint(2.38431262969971,2.03659462928772,1.02624261379242);
 addPoint(2.36153221130371,2.12736034393311,0.964407861232758);
 addPoint(2.33197236061096,2.22583508491516,0.916022539138794);
 addPoint(2.29541444778442,2.33240151405334,0.882726907730103);
 addPoint(2.25163984298706,2.44744205474854,0.866161227226257);
 addPoint(2.20042967796326,2.57133960723877,0.867965519428253);
 addPoint(2.14156532287598,2.70447707176209,0.889780104160309);
 addPoint(2.07482814788818,2.84723663330078,0.933245062828064);
 addPoint(2,3,1);
 addPoint(1.91902959346771,3.28923654556274,0.999981701374054);
 addPoint(1.83423256874084,3.5271909236908,0.999854266643524);
 addPoint(1.74593603610992,3.7165949344635,0.999507963657379);
 addPoint(1.65446841716766,3.86018323898315,0.998833537101746);
 addPoint(1.56015837192535,3.96069049835205,0.997722029685974);
 addPoint(1.46333289146423,4.02084827423096,0.996063530445099);
 addPoint(1.3643205165863,4.04339170455933,0.993749022483826);
 addPoint(1.26344931125641,4.03105354309082,0.990669071674347);
 addPoint(1.16104757785797,3.98656892776489,0.986714601516724);
 addPoint(1.05744278430939,3.91266942024231,0.981775760650635);
 addPoint(0.952963352203369,3.8120903968811,0.975743651390076);
 addPoint(0.847937047481537,3.6875638961792,0.968508541584015);
 addPoint(0.742692053318024,3.54182434082031,0.959961295127869);
 addPoint(0.637556672096252,3.37760639190674,0.949992716312408);
 addPoint(0.532858550548553,3.19764256477356,0.938493311405182);
 addPoint(0.428925722837448,3.00466585159302,0.925353586673737);
 addPoint(0.326086342334747,2.80141067504883,0.910464286804199);
 addPoint(0.224668622016907,2.59061145782471,0.893716335296631);
 addPoint(0.125000402331352,2.37500095367432,0.875000178813934);
 addPoint(0.0274096131324768,2.15731239318848,0.854206264019012);
 addPoint(-0.067775622010231,1.94027984142303,0.831225454807282);
 addPoint(-0.160227224230766,1.72663700580597,0.805948436260223);
 addPoint(-0.249617144465446,1.51911759376526,0.778265833854675);
 addPoint(-0.335617333650589,1.3204550743103,0.748068273067474);
 addPoint(-0.417899817228317,1.13338315486908,0.715246379375458);
 addPoint(-0.496136516332626,0.960635602474213,0.679690897464752);
 addPoint(-0.56999933719635,0.804945886135101,0.64129239320755);
 addPoint(-0.639160335063934,0.669047713279724,0.599941611289978);
 addPoint(-0.703291475772858,0.555674731731415,0.555529117584229);
 addPoint(-0.762064635753632,0.467560619115829,0.507945597171783);
 addPoint(-0.815151810646057,0.407438933849335,0.457081735134125);
 addPoint(-0.862224996089935,0.378043353557587,0.402828127145767);
 addPoint(-0.902956187725067,0.382107555866241,0.345075488090515);
 addPoint(-0.937017261981964,0.42236515879631,0.283714443445206);
 addPoint(-0.964080214500427,0.501549780368805,0.218635633587837);
 addPoint(-0.983817040920258,0.622395038604736,0.149729743599892);
 addPoint(-0.995899617671967,0.787634611129761,0.0768874064087868);
 addPoint(-1,1.00000214576721,-7.15255907834944E-7);
end;


{------------------------------------------------------------------}
{  Function to do an arctan                                        }
{------------------------------------------------------------------}
function ArcTan(X, Y : glFloat) : glFloat;
asm
  FLD     Y
  FLD     X
  FPATAN
  FWAIT
end;


{----------------------------------------------------------}
{  Initialise Demo. Loads textures and creates objects     }
{----------------------------------------------------------}
procedure ShowLoading;
begin
  glTranslatef(0.0,0.0,-4);

  // Background image
  LoadTexture('images/freewill.jpg', ProgressTex, false);
  glBindTexture(GL_TEXTURE_2D, ProgressTex);
  glBegin(GL_QUADS);
    glTexCoord2f(0, 0); glVertex3f(-1,-0.125, 0);
    glTexCoord2f(1, 0); glVertex3f( 1,-0.125, 0);
    glTexCoord2f(1, 1); glVertex3f( 1, 0.125, 0);
    glTexCoord2f(0, 1); glVertex3f(-1, 0.125, 0);
  glEnd();

  ElapsedTime :=0;
  Inc(Stage);
end;


{----------------------------------------------------------}
{  Initialise Demo. Loads textures and creates objects     }
{----------------------------------------------------------}
procedure Init;
begin
  glTranslatef(0.0,0.0,-10);

  // Background image
  glBindTexture(GL_TEXTURE_2D, ProgressTex);
  glBegin(GL_QUADS);
    glTexCoord2f(0, 0); glVertex3f(-2,-0.25, 0);
    glTexCoord2f(1, 0); glVertex3f( 2,-0.25, 0);
    glTexCoord2f(1, 1); glVertex3f( 2, 0.25, 0);
    glTexCoord2f(0, 1); glVertex3f(-2, 0.25, 0);
  glEnd();

  SphereQuadratic := gluNewQuadric();		       // Create A Pointer To The Quadric Object (Return 0 If No Memory) (NEW)
  gluQuadricNormals(SphereQuadratic, GLU_SMOOTH);	       // Create Smooth Normals (NEW)
  gluQuadricTexture(SphereQuadratic, true);	       // Create Texture Coords (NEW)

  initLazerCurve;
  setLength(vCurves,1);
  initCurve;
  LoadDemoTextures;

  // Init 3D Logo
  g_VolitionLogo := TAnimate.create('images\volitionlogo.txt',24,true,true);
  g_RotLogo := 0.0;


  ElapsedTime :=0;
  Inc(Stage);
end;

end.

